#!/usr/bin/perl

#---------------------------------------------------------
#GPS Device Sample Code by J.P. Stewart (jp@jpstewart.org)
#This Code requires a Serial port GPS device which is NMEA-0183 compliant

#This code can be freely used as long as this section is in tact.
#---------------------------------------------------------


#Used later to calculate current time
use Time::Local;
use IO::Handle;

$|=1;

#Check the first argument to see if a comm port to read from was specified
if ($ARGV[0] !~ m/./i) {

	#If no comm port was specified, then print the usage and exit.
	print "Usage: gps.pl <COMX> [<output file>]\r\n";
	exit;
}

#Set the comm port to be used and confirm to the user that it was recieved
$COMMPORT = $ARGV[0];
print "Reading $COMMPORT...\r\n";

#check the second argument to see if a filename was specified
#NOTE: You may want to do more robust string matching here than I do.
if ($ARGV[1] =~ m/./) {

	#Set the output file to be what the user specified and 
# confirm to the user that it was recieved
	$fileparam = $ARGV[1];
	print "Storing to $fileparam...\r\n";
}

#Open a handle to the port provided. Or exits with the failure if an error occurs.
#Note: This may only work in Win2000 and higher, 
#      check to see if this will work in your OS.
#      If not, just replace this line with one that assigns a file handle to a COM port.
#      Or, alternatively, you can replace with any code that gets string data from the
#	COM port.
#      If in doubt, see one of these modules:
# 	    GPS::Serial Module: 
#		http://search.cpan.org/~srezic/perl-GPS-0.14/Serial.pm
#	    Win32::SerialPort Module: 
#		http://search.cpan.org/dist/Win32-SerialPort/lib/Win32/SerialPort.pm
open( PORT, "<$COMMPORT" ) or die "Can't open $COMMPORT: $!";

#Reads one line at a time from the COMM port in an infinite loop, 
# you can use CTRL+C to exit
while (<PORT>) {

	#Check to see if this "sentence" contains location information, 
# if so, process it. (checks if the line starts with "$GPGGA".)
	if ($_ =~ m/^\$GPGGA/i) {

		#Create an array of fields from the Comma Delimited sentence
		#See the text for a table of the value definitions.
		@info = split(/,/);

		#I am only interested in results of 3 or more satellites. 
		#(This can also be replaced with ($info[6] > 0) if so desired)
		if ($info[7] >= 3) {

			#Convert the latitude from NMEA-0183 format to decimal 
# (using decimal place as a reference)
			$lat = substr($info[2],0,(index($info[2],"\.") - 2)) + 
       (substr($info[2],(index($info[2],"\.") - 2)) / 60.0);

			#Convert the llongitude from NMEA-0183 format to decimal 
# (using decimal place as a reference)
			$long = substr($info[4],0,(index($info[4],"\.") - 2)) + 
        (substr($info[4],(index($info[4],"\.") - 2)) / 60.0);
				
			#Assign negative values if necessary
			$lat = "-".$lat if ($info[3] =~ m/S/i);
			$long = "-".$long if ($info[5] =~ m/W/i);

			#Convert NMEA-0183 time to current time by converting GMT time
# provided by satellite to Local time
			@gma = gmtime(time);
			($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = 
			  localtime(timegm(substr($info[1],(index($info[1],"\.") - 2),2),
  substr($info[1],(index($info[1],"\.") - 4),2),
  substr($info[1],0,(index($info[1],"\.") - 4)),
  $gma[3],$gma[4],$gma[5]));				
			$sec = "0".$sec if ($sec < 10);
			$min = "0".$min if ($min < 10);
			$mon++;
			$year += 1900;

			#Check to see if we have a data point already
			if ($AveragedLatitude =~ m/./) {

				#Found an existing data point, check to see if the new data 
#  point is greater than 45ft in either direction
				#Feel free to tweak this link based on the background I 
#  included in the text.
				if ((abs($lat - $AveragedLatitude) > 0.0001) || 
  (abs($long - $AveragedLongitude) > 0.0001)) {
					
					#New data point is greater than 45 feet away.
					#Check to see if we are logging to a file.
					if ($fileparam) {
						
						#If so, open the file for appending and 
# print out the results.
						open(OUT,">>$fileparam");
						print OUT "$AveragedTime,".
          "$AveragedLatitude,".
          "$AveragedLongitude,".
          "$AveragedAltitude\r\n";
						close(OUT);
					} else {
						
						#No file parameter given, 
#  just print out the results.
						print "$AveragedTime,".
      "$AveragedLatitude,".
      "$AveragedLongitude,".
      "$AveragedAltitude\r\n";
					
					}

					#After the previous data has been saved or printed, 
# reset the stored datapoint to the new one
					$AveragedTime = "$mon/$mday/$year $hour:$min:$sec";
					$AveragedLatitude = $lat;
					$AveragedLongitude = $long;
					$AveragedAltitude = $info[9];
				} else {
					#The latest datapoint is too close to the last one.

					#Uncomment this section to show averaged results. 
					#print "\<$AveragedTime,".
#      "$AveragedLatitude,".
#      "$AveragedLongitude,".
#      "$AveragedAltitude\>\r\n" if (!($fileparam));
					
					#Average the new data point in with the others 
# and reset the time.

					$AveragedTime = "$mon/$mday/$year $hour:$min:$sec";
					$AveragedLatitude = 
($AveragedLatitude + $lat) / 2;
					$AveragedLongitude = 
($AveragedLongitude + $long) / 2;
					$AveragedAltitude = 
($AveragedAltitude + $info[9]) / 2;
				}
			} else {	

				#There is no current datapoint, so save the current values.
				$AveragedTime = "$mon/$mday/$year $hour:$min:$sec";
				$AveragedLatitude = $lat;
				$AveragedLongitude = $long;
				$AveragedAltitude = $info[9];
			}

				
		}
	}
}
