#!/usr/bin/perl

#---------------------------------------------------------
#GPS Device Sample Code by J.P. Stewart (jp@jpstewart.org)
#This code creates a static web page (as well as 2 other possible files) based on GPS 
# location input.
#There are two input methods. The first is from a file, and the second is via STDIN.
#The format of the file must be lines like so:
#       "[time],[latitude],[longitude],[altitude]"
#These lines can then repeat in the same format until the end of the file.
#
#Luckily, this is also the exact format that my initial GPS code outputs.
#This gives you a few options to choose from:
# 1. You can store the output from the GPS code in a file (see its usage on how to do 
#    that) and then process the file later with this script
#	usage: perl MakeHTML.pl SavedGPSFile.txt
# 2. You can pipe the output from my initial GPS code directly into this script.
#	usage: perl GPS.pl|perl MakeHTML.pl
# 3. You can use either method 1. or 2. with your own code provided you follow the format
#    above.
#
#Additionally, this script will use MapPoint if it is installed to make a map showing 
# your route and will also output both an image AND a MapPoint ptm file linked in the 
# HTML.
#You can provide the name of the mappoint PTM file as the second argument to this script.
# (The default is the current time)
#If MapPoint is not available, then a static text file which is the same as Example 2
# will be created.
#
#In either case, the name of the HTML file to create is stored below. The default is
# MyLocations.html.
#
#This code can be freely used as long as this section is in tact.
#---------------------------------------------------------

$HTMLPage = "MyLocations.html";

# End of user configurable section
#_________________________________


#Used later to calculate current time
use Time::Local;

#Autoflush STDOUT buffer
$|=1;

#Check the first argument to see if a file to read from was specified 
#(Otherwise reads from STDIN).
if ($ARGV[0] =~ m/./i) {
	$input = "FILE";
	open($input,"<$ARGV[0]");
} else {
	$input = "STDIN";
}

#Find the current directory
$currentdir = $ENV{PATH_TRANSLATED};
$currentdir =~ s/$0//i;

#This will attempt to create a MapPoint Object. If it fails, it sets a variable which is 
# checked everywhere to see if MapPoint is available.
require Win32::OLE or $noole = 1;
$mappoint = Win32::OLE->new("MapPoint.Application") or $nomappoint = 1 if (!($noole));

#If MapPoint IS installed, do some more initialization
if (!($nomappoint)) {

	#Check the second argument to see if a map output file was specified (Otherwise 
	# makes [time].ptm where [time] is the current time in seconds in the current 
	# directory). 
	if ($ARGV[1] =~ m/./i) {
		$outputmapfile = "$ARGV[1]";

		#Looks to see if a full path was supplied and if not, we specify the 
		# current directory.
		$outputmapfile = $currentdir.$outputmapfile if 
			(($outputmapfile =~ m/\//) || ($outputmapfile =~ m/\\/));
	} else {
		
		#The default map output is the current time in the current directory.
		$outputmapfile = $currentdir.time.".ptm";
	}

	#Currently the only way to get a map image from MapPoint is via the clipboard.
	use Win32::Clipboard;
	$CLIP = Win32::Clipboard();

	#This creates a blank map and sets the altitude to 10. You can adjust this if you
	# want the final images more zoomed in or out.
	$map = $mappoint->NewMap();
	$map->{Altitude} = 10;

	#The image that is saved is currently in BMP form. The name is taken from 
	# whatever the map file is. Just the extension is replaced.
	$outputimage = $outputmapfile;
	$outputimage =~ s/\.\S+?$/.bmp/;

	#This is the master layout for the HTML when MapPoint is installed. Feel free to 
	# adjust this how you like. However, variables like: !!Cur_Long!! etc. MUST 
	# remain in the string.
	$HTMLOut = "<html><head><title>My Last Locations</title></head><body>\n".
	      "<table border=1><tr><td><h2>My Current Location is:<br><center>".
	      "<a href=\"http://mapper.acme.com/?lat=!!Cur_Lat!!&long=".
	      "!!Cur_Long!!&scale=10&width=4&height=3&dot=Yes\">".
	      "!!Cur_Lat!! Latitude<br>!!Cur_Long!! Longitude</a></center><br>".
	      "I was at this location at this time:<br><center>!!Cur_Time!!</center>".
	      "</h2></td></tr></table><br><b>My previous locations have been:</b>\n<br>".
	      "!!IMAGE!!<br><a href=\"$outputmapfile\">MapPoint File containing route".
	      "</a>\n</body></html>";
} else {

	#This is the master layout for the HTML when there is no MapPoint. Feel free to 
	# adjust this how you like. However, variables like: !!Cur_Long!! etc. MUST 
	# remain in the string.
	$HTMLOut = "<html><head><title>My Last Locations</title></head><body>\n".
	      "<table border=1><tr><td><h2>My Current Location is:<br><center>".
	      "<a href=\"http://mapper.acme.com/?lat=!!Cur_Lat!!&long=".
	      "!!Cur_Long!!&scale=10&width=4&height=3&dot=Yes\">".
	      "!!Cur_Lat!! Latitude<br>!!Cur_Long!! Longitude</a></center><br>".
	      "I was at this location at this time:<br><center>!!Cur_Time!!</center>".
	      "</h2></td></tr></table><br><b>My previous locations have been:</b>\n<br>".
	      "<UL>!!LIST_ITEMS!!</UL>\n</body></html>";
}

#This will read one line at a time of either the file or STDIN.
while (<$input>) {

	#Parse the comma delimited line into seperate variables.
	($time,$lat,$long,$alt) = split(/,/);

	#Check to make sure that the two we really want are there.
	if (($lat) and ($long)) {

		#Check again to see if MapPoint is available
 		if (!($nomappoint)) {

			#Since you have to provide a start AND end when drawing a line in 
			# MapPoint, we need to buffer one location for the next time.
			if ($OldMapPointlocation) {

				#Pass the coordinates to MapPoint and get a location
				# object.
				$NewMapPointlocation = $map->GetLocation($lat,$long);

				#Change the view of the map to focus on the new point.
				$NewMapPointlocation->GoTo();

				#Draw a line from the previous point to the new point on
				# the map.
				$map->shapes->AddLine(
					$OldMapPointlocation,$NewMapPointlocation);

				#If we are proccessing from STDIN, we need to save what 
				# we have.
				if ($input eq "STDIN") {
					
					#Save the MapPoint file
					$map->SaveAs("$outputmapfile",0,false) ;

					#Copy an image of the map to the clipboard, get 
					# time image from the clipboard, and write it to 
					# a file.
					$map->CopyMap;
					$image  = $CLIP->GetBitmap();
					open(IMAGE,">$outputimage");	
					binmode IMAGE;
					print IMAGE $image;
					close(IMAGE);

					#Replace the parts of the HTML Layout with the 
					# correct information
					$HTMLOut =~ s/!!Cur_Lat!!/$lat/gi;
					$HTMLOut =~ s/!!Cur_Long!!/$long/gi;
					$HTMLOut =~ s/!!Cur_Time!!/$time/gi;
					$HTMLOut =~ 
					      s/!!IMAGE!!/<img src=\"$outputimage\">/gi;

					#Pring the location that was just processed.
					print "Processed: $lat,$long\n";

					#Write out the HTML file.
					open(HTML,">$HTMLPage");
					print HTML $HTMLOut;
					close(HTML);
				}

				#Swap the new location to the old location
				$OldMapPointlocation = $NewMapPointlocation;
			} else {
				
				#This should only get called the first time.
				$OldMapPointlocation = $map->GetLocation($lat,$long);
			}

		} else {
			#Else, there is no MapPoint, so make a static HTML file from 
			# text.
			
			#Add the current coordinates and time to an array.
			push(@locs,"$lat,$long,$time");

			#If we are proccessing from STDIN, we need to save what 
			# we have.
			if ($input eq "STDIN") {

				#Replace the parts of the HTML Layout with the 
				# correct information
				$HTMLOut =~ s/!!Cur_Lat!!/$lat/gi;
				$HTMLOut =~ s/!!Cur_Long!!/$long/gi;
				$HTMLOut =~ s/!!Cur_Time!!/$time/gi;
				
				#This builds the list of past locations from the array 
				# built above.	
				$list = "";
				foreach (reverse @locs) {
					my($ilat,$ilong,$itime) = split /,/;

					#Each location has an Acme Mapper Link.
					$list = $list."<LI><a href=".
						"\"http://mapper.acme.com/?lat=".$ilat.
						"&long=".$ilong."&scale=10&width=".
						"4&height=3&dot=Yes\">".
						"$ilat, $ilong</a> \@ $itime\n";
				}

				#Replace the previous Unordered List withe the one that
				# was just built above.
				$HTMLOut =~ s/<UL>.+?<\/UL>/<UL>$list<\/UL>/gis;

				#Pring the location that was just processed.
				print "Processed: $lat,$long\n";

				#Write out the HTML file.
				open(HTML,">$HTMLPage");
				print HTML $HTMLOut;
				close(HTML);
			}
		}
	}
}

close($input);

#In order to save time, when processing a file vs. STDIN, ALL the saving is done at the
# end rather than in the middle. This checks to ensure that we are NOT processing from 
# STDIN
if (!($input eq "STDIN")) {

	#Check to see if MapPoint is available.
	if (!($nomappoint)) {


		#Save the MapPoint file
		$map->SaveAs("$outputmapfile",0,false) ;

		#Copy an image of the map to the clipboard, get 
		# time image from the clipboard, and write it to 
		# a file.
		$map->CopyMap;
		$image  = $CLIP->GetBitmap();
		open(IMAGE,">$outputimage");	
		binmode IMAGE;
		print IMAGE $image;
		close(IMAGE);

		#Replace the parts of the HTML Layout with the 
		# correct information
		$HTMLOut =~ s/!!Cur_Lat!!/$lat/gi;
		$HTMLOut =~ s/!!Cur_Long!!/$long/gi;
		$HTMLOut =~ s/!!Cur_Time!!/$time/gi;
		$HTMLOut =~ s/!!IMAGE!!/<img src=\"$outputimage\">/gi;
	
	#Otherwise generate plain HTML.
	} else {
			
		#Replace the parts of the HTML Layout with the 
		# correct information		
		$HTMLOut =~ s/!!Cur_Lat!!/$lat/gi;
		$HTMLOut =~ s/!!Cur_Long!!/$long/gi;
		$HTMLOut =~ s/!!Cur_Time!!/$time/gi;
			
		#This builds the list of past locations from the array built above.
		$list = "";
		foreach (reverse @locs) {
			my($ilat,$ilong,$itime) = split /,/;
			#Each location has an Acme Mapper Link.
			$list .= "<LI><a href=\"http://mapper.acme.com/?lat=".$ilat.
			         "&long=".$ilong."&scale=10&width=4&height=3&dot=Yes\">".
			         "$ilat, $ilong</a> \@ $itime\n";
		}

		#Replace the previous Unordered List withe the one that was just built.
		$HTMLOut =~ s/!!LIST_ITEMS!!/$list/gi;
	}

	#Write out the HTML file.
	open(HTML,">$HTMLPage");
	print HTML $HTMLOut;
	close(HTML);
}
