;	Carbot ITPS (ITuner Power Sequencer)
;
;	The original code was derived from the ITPS firmware delivered by ITuner.
;	It has been rewritten for new logic.
;
;	Written by Zack Gainsforth
;	26 March 2004
;
;	
; Pin definitions:
;
; GP0 = IGNITION		input	1 = ignition turned on
;					Connected to Accessory on in car.
; GP1 = POWEROUT		output	1 = turn power on
;					Connected to transistor in IPTS
; GP2 = COMPUTERISON		input	1 = Mini-ITX is on.
;					Connected to +5V rail on Mini-ITX power supply			
; GP3 = N.C.
; GP4 = COMPUTERSHOULDBEON	output	1 = Mini-ITX should be on, 0 = off.
;					Connected to the carrier detect (CD) pin on
;					COM 2 in the Mini-ITX.
; GP5 = N.C.
	
;-------------------------------------------------------------------------------

; Definitions for people:

;Tick:
;	This MCU measures time in 250ms increments.  This increment is called a tick.
;	4 ticks make a second, 60 seconds/minute, 60 minutes/hour.

;Seven bytes provides a time structure.
;	Byte 1) Time.Ticks
;	Byte 2) Time.Seconds
;	Byte 3) Time.Minutes
;	Byte 4) Time.Hours
;	This allows the MCU to keep track of time with a 1/5th second resolution up
;	to 254 hours. (A couple weeks.)
;	Byte 5) Alarm.Seconds
;	Byte 6) Alarm.Minutes
;	Byte 7) Alarm.Hours
;	The alarms can be compared against a time structure to determine if we've gone
;	past the alarm time.
;
;	Notice, the alarm portion is not strictly required for all functions.  It is
;	only required if you wish to compare the time against an alarm.

;Snooze:
;	The Mini-ITX computer is off most of the time that the car is off.  However,
;	it periodically needs to boot to sniff for a wi-fi connection, download e-mail,
;	etc.  Therefore, there is a *snooze period* which is how long to wait between 
;	computerboot cycles while the car is off.

;Budget:
;	The computer uses a significant amount of current when booted.
;	Therefore, even if it is only booted for a couple minutes, it can drain a car
;	battery after a number of boots.  Therefore, there is a time budget.  It is 
;	the accumulated time that the computer can be on between ignition cycles.
;	It is assumed that an ignition cycle = a fully recharged car battery.

;-------------------------------------------------------------------------------

	LIST	p=12F629 	;Target processor
	#include "P12F629.INC"	;Header file for target processor

;-------------------------------------------------------------------------------

; Equates


IGNITION		equ 0x00	;Input
POWEROUT		equ 0x01	;Output
COMPUTERISON		equ 0x02	;Input
NC1			equ 0x03	;No Connection
COMPUTERSHOULDBEON		equ 0x04	;Output
NC2			equ 0x05	;No Connection
	
STABILIZECARPOWERDELAY		equ .16	;wait 16 ticks (4 seconds) after the ignition turns
					;the car on before doing anything in order to let
					;the power levels stabilize.
STABILIZEITPSPOWERDELAY		equ .4	;wait 5 ticks (1 second) after turning on power
					;within the ITPS board in order to let power stabilize.
STABILIZEMINIITXPOWERDELAY	equ .20 ;wait 20 ticks (5 seconds) after giving power to the
					;computer before checking if it is really turned
					;on.  It takes a bit for the computer's powersupply
					;to stabilize.

;-------------------------------------------------------------------------------

; Register variables

;Variables used in Delay250ms
cntr	equ	0x20	;state counter
cntr2	equ	0x21	
waitl	equ	0x22	;delay counter low
waitm	equ	0x23	;delay counter mid
waith	equ	0x24	;delay counter high

;A time budget ensure we don't suck the car battery dry.  The MCU needs to
;keep track of how much time the computer has been on (amp-hours) since the
;last ignition cycle.
;The MCU also needs to know how long it can allow the computer to be on.
;(The computer uses on average ~ 1 amp or less.  1.5 hours, keeps the total
;budget under 1.5 amp-hours at 12-14 V)
;There are two alarms:	1) Budget alarm kills the Mini-ITX when it goes overbudget.
;						2) No boot alarm doesn't boot the Mini-ITX when it only has
;							five minutes left until it goes overbudget.

;The following is the budget time structure (with budget alarm, and no boot alarm): 0x25 - 0x2E
budgettime	equ 0x25

;Standby waits for a reason to turn on the computer.
;It needs to know how long it has been waiting.
;The alarm signifies how long it should wait before arbitrarily starting the computer.

;The following is the standby time structure (with alarm): 0x2F - 0x35
standbytime	equ 0x2F	;standby counter handle

;DelayTicks is a parameter passed to the delay function to tell it how many ticks to delay.
DelayTicks	equ 0x36

;BootType describes which kind of bootup this is:
BootType	equ 0x37
UNBOOTED		equ 0	;The computer is supposed to be off!
BOOTIGNITION	equ 1	;The computer is booting because the driver turned on the car.
BOOTSNOOZE		equ 2	;The computer is booting because it is waking up from a snooze.

;DecrementCounter is a counter used by DecrementTime for decrementing the budget counter.
;The budget counter needs to be decremented at a faster rate when the battery is charging
;than it increments when the car is turned off and the Mini-ITX is running.
DecrementCounter	equ 0x38

;DecrementTimeStruct holds the FSR register pointing to the time structure being 
;used by DecrementTime.
DecrementTimeStruct	equ 0x39

;Internal variable used by CheckAlarm.  It keeps track of how far the alarm is from
;the beginning of the time structure (alarm 1 is 4 bytes beyond, alarm 2 is 7 bytes beyond.)
AlarmOffset			equ 0x40
AlarmCounter		equ 0x41
ALARM1OFFSET		equ 0x03	;Alarm1.x is 3 bytes after Time.x
ALARM2OFFSET		equ 0x06	;Alarm2.x is 6 bytes after Time.x

;-------------------------------------------------------------------------------

	; BOOT UP ENTRY POINT
	; This is the first instruction executed when power is first supplied to the microchip.
	org	0
	goto	Initialize
	
	; INTERRUPT HANDLER
	org	4
	retfie

;-------------------------------------------------------------------------------

;Initialize:
;This is the boot up code.
;Set the oscillator speed, initialize input/output pins, initialize alarms, etc.

Initialize
	;Set the oscillator speed.
	banksel	OSCCAL	;get oscillator calibration
	call	0x3ff
	movwf	OSCCAL & 0x7f
	
	;Terminate GP5.
	movlw	b'100000'	;enable weak pull-up on
	movwf	WPU & 0x7f	;GP5 (doesn't seem to work...)
	
	;Set the option register.
	;1) Enable weak pull-ups (enable the WPU register)
	;2) Set to timer0 internal clock, positive edge.
	;3) Assign the prescaler to WDT, 1:32 -> ~0.5s
	movlw	b'00001101'
	movwf	OPTION_REG & 0x7f

	;Clear the watchdog
	clrwdt

	;The ITPS is just starting.  Assume a full car battery and
	;set the power budget as unused.
	movlw	budgettime	;FSR = budgettime
	movwf	FSR
	call ClearTime

	;Initialize the budget alarm to 1 hour 30 minutes.
	movlw	.0		;(budgettime)Alarm.Seconds = 0
	movwf	budgettime+4
	movlw	.30		;(budgettime)Alarm.Minutes = 30
	movwf	budgettime+5
	movlw	.1		;(budgettime)Alarm.Hours = 1
	movwf	budgettime+6

	;Initialize the no boot alarm to 1 hour 25 minutes.
	movlw	.0		;(budgettime)Alarm.Seconds = 0
	movwf	budgettime+7
	movlw	.25		;(budgettime)Alarm.Minutes = 25
	movwf	budgettime+8
	movlw	.1		;(budgettime)Alarm.Hours = 1
	movwf	budgettime+9

	;Initialize the standby alarm to 2 hours.
	movlw	.0		;(standbytime)Alarm.Seconds = 0
	movwf	standbytime+4
	movlw	.0		;(standbytime)Alarm.Minutes = 0
	movwf	standbytime+5
	movlw	.2		;(standbytime)Alarm.Hours = 2
	movwf	standbytime+6
	
	;Consider the computer unbooted.
	movlw	UNBOOTED	;BootType = UNBOOTED
	movwf	BootType
	
	;Set which pins are inputs and which are outputs (in the TRISIO register).
	banksel	TRISIO
	movlw	0			;Set all bits to output first (0 means output)
	movwf	TRISIO
	bsf	TRISIO, IGNITION	;Ignition is an input bit. (1 means input)
	bsf	TRISIO, COMPUTERISON	;COMPUTERISON is an input feed from the computer
					;high means the computer is on, low means not.

	;Now set the values on the output pins (in the GPIO register).
	banksel	GPIO
	movlw	0			;Power out = off
	movwf	GPIO
			
	;Turn the comparator off.
	movlw	7
	movwf	CMCON

;-------------------------------------------------------------------------------

;Standby:
;This is the loop that runs while the car is turned off.

;Logic:
;	Count time into the standby variables until either:
;	1) The car is turned on from the ignition.
;	2) The device's snooze period expires.

Standby

	;The standby is just starting.  Reset the standby counters.
	movlw	standbytime		;FSR = standbytime
	movwf	FSR
	call ClearTime
		
standbywait	call	Delay250ms	;wait one tick

	;Increment the standby time by one tick.
	movlw	standbytime
	movwf	FSR
	call IncrementTime

	;See if almost overbudget (no boot alarm triggered).  
	;If so, don't check the standby alarm as it will boot the
	;computer if the snooze period is up.
	movlw	budgettime		;FSR = budgettime
	movwf	FSR
	movlw	ALARM2OFFSET		;Check the second alarm (no-boot)
	call	CheckAlarm
	xorlw	0			;if w = 0 ...
	btfss	STATUS, 2		;...continue
	goto skipsnoozecheck
	
	;Now see if a standby alarm is triggered (i.e. has it been a snooze period?)
	;If so, boot the computer.
	movlw	standbytime		;FSR = standbytime
	movwf	FSR
	movlw	ALARM1OFFSET		;Check the first (and only) alarm.
	call CheckAlarm
	xorlw	0			;if w = 0 ...
	btfss	STATUS, 2		;...continue
	goto	BootComputerSnooze	;otherwise wake up the computer
	
skipsnoozecheck
	;Check if the user has turned the car on via the ignition
	btfsc	GPIO, IGNITION		;Ignition = ?
	goto BootComputerIgnition	;Wake up the computer
	
	;No reasons exist to turn on the computer.  Wait some more.
	goto standbywait


;-------------------------------------------------------------------------------

;BootComputer:
;This is called whenever it is time to turn the computer on.
;
;Logic:
;	The car can be turned on when:
;	1) The car is turned on from the ignition.
;	2) The device's snooze period expires.
;
;The purpose of BootComputer is to turn on the power circuits with delays
;to allow power levels to stabilize before proceeding.  This avoids spikes hitting
;sensitive gear further on.
;
;BootComputer has two entry points:
;	BootComptuterSnooze is called when the computer is waking up from a snooze.
;		The car isn't really on.
;	BootComputerIgnitino is called when the driver turns the car on.

BootComputerIgnition
	;First store the boot type to a variable for later reference.
	movlw	BOOTIGNITION	;BootType = BOOTIGNITION
	movwf	BootType
	
	;The power just started, wait for the power to stabilize before doing anything.
	;In the case where we are unsnoozing, this delay is useless, but harmless.
	movlw	STABILIZECARPOWERDELAY
	movwf	DelayTicks
	call	Delay

	;Make sure the car is still on.  Sometimes the starter revs and the car
	;doesn't catch.  Don't turn the engine on yet.
	btfss	GPIO, IGNITION		;Ignition = ?
	goto Standby			;The computer isn't on now, so back to standby.

	;If the car is still on, then proceed to booting the Mini-ITX.
	goto	BootComputer

BootComputerSnooze
	;Store the boot type to a variable for later reference and then proceed
	;to booting the Mini-ITX.
	movlw	BOOTSNOOZE	;BootType = BOOTSNOOZE
	movwf	BootType
	goto	BootComputer

BootComputer
	
	;Activate the power within the ITPS.
	;This gives power to the Mini-ITX, and turns it on.
	bsf	GPIO, POWEROUT		;Bit 2 controls the powerout pin.
	
	;Now wait a bit for power to stabilize within the ITPS board before continuing.
	movlw	STABILIZEITPSPOWERDELAY
	movwf	DelayTicks
	call	Delay
	
	;At this point ITPS is on.  Go to the monitoring function.
	
	;If this is an ignition cycle, goto MonitorOnCycleIgnition
	movlw	BOOTIGNITION	;if BootType = BOOTIGNITION
	xorwf	BootType, 0
	btfsc	STATUS, 2		;Zero flag = zero if BootType = BOOTIGNITION
	goto	MonitorOnCycleIgnition	;If equal, goto MonitorOnCycleIgnition
	goto	MonitorOnCycleSnooze	;If not equal, it must be snooze.
	
;-------------------------------------------------------------------------------

;MonitorOnCycleIgnition:
;This runs while the computer is on.  It takes responsibility for deciding when
;the power should be killed to the computer after the user turns the car off.
;
;Logic:
;	In BOOTIGNITION, hold the COMPUTERSHOULDBEON pin hi to tell the computer that it
;	should be on.  This lets the driver have his carbot while he drives the car.
;	Watch for him to turn off the power to the car.  When he does, change
;	COMPUTERSHOULDBEON to low, and shutdown when the computer COMPUTERISON(input)
;	goes low meaning the computer has fininshed shutting down.

MonitorOnCycleIgnition

	;Since the car is on, the computer should stay on too.
	;Set the COMPUTERSHOULDBEON(output) pin high to communicate this to the
	;computer.  This pin maps to the CD pin on COM2.
	banksel GPIO
	bsf	GPIO, COMPUTERSHOULDBEON
	
checkignitionon
	;Is the ignition still on?
	btfss	GPIO, IGNITION
	goto	ShutdownComputer	;Ignition is not still on.  Therefore, shutdown
					;the computer

	;The computer is supposed to be on, but is it?
	;If not, power cycle it so it can reboot.
	btfss	GPIO, COMPUTERISON
	goto	HardShutdown		;If COMPUTERISON = 0, then shutdown computer.

	;Now wait a tick, and check again.
	call Delay250ms

	;Decrement the budget time 6X since the car battery is being
	;recharged while the car is running.
	movlw	budgettime		;FSR = budgettime
	movwf	FSR
	movlw	.6			;Decrement at 6X
	movwf	DecrementCounter
	call	DecrementTime
	
	;Check again!
	goto checkignitionon	
	
;-------------------------------------------------------------------------------

;MonitorOnCycleSnooze:
;This runs while the computer is on.  It takes responsibility for deciding when
;the power should be killed to the computer after the computer finishes
;its snooze tasks.
;
;Logic:
;	In BOOTSNOOZE, hold the COMPUTERSHOULDBEON(output) pin low to tell it that it should shut
;	down as soon as it has no tasks.  This minimizes battery usage.  Then kill power and go
;	back to standby as soon as COMPUTERISON(input) goes low.

MonitorOnCycleSnooze

	;Since the car is not on, minimize the computer's on time.
	;Have it do its tasks and then shut down.
	
	;Now wait a bit for power to stabilize within the Mini-ITX before continuing.
	;This gives it a chance for the power to really come on within the computer
	movlw	STABILIZEMINIITXPOWERDELAY
	movwf	DelayTicks
	call	Delay

	;There isn't any monitoring to do now, because the computer
	;is supposed to snooze again asap.  Go to the shudown routine.
	goto ShutdownComputer
		
;-------------------------------------------------------------------------------

;ShutdownComputer:
;This turns the computer off.
;
;Logic:
;	1) Ensure the computer knows it is supposed to shut down. (COMPUTERSOULDBEON pin)
;	2) Kill power when the computer actually shuts down.
;	3) Return to standby mode.

ShutdownComputer

	;Set the COMPUTERSHOULDBEON(output) pin to low to tell the computer to
	;shutdown.  The pin maps to the CD pin on COM2.
	banksel GPIO
	bcf	GPIO, COMPUTERSHOULDBEON
	
checkshutdown
	;Now see if the computer has shutdown.
	btfss	GPIO, COMPUTERISON
	goto continueshutdown			;Computer is now off.  Therefore, proceed
						;with the shutdown.

	;Check the ignition too.  Sometimes the car turns off for a brief while,
	;and the computer has not shut down.  (For example, the guy's engine quits
	;at a stop sign, or he stops briefly at Starbucks.  No need to reboot
	;the computer in these cases.)
	btfsc	GPIO, IGNITION
	goto	MonitorOnCycleIgnition

	;The computer is still on.  Wait for it to finish shutting down.
	;Now wait a tick, and check again.
	call Delay250ms
	
	;The computer only shuts down when the car is off.
	;Mark this tick in the power budget.
	movlw	budgettime
	movwf	FSR
	call IncrementTime	
	
	;See if overbudget.  If so, just kill the power to the Mini-ITX.
	movlw	budgettime		;FSR = budgettime
	movwf	FSR
	movlw	ALARM1OFFSET		;Check the first alarm (overbudget).
	call	CheckAlarm
	xorlw	0			;if w = 0 ...
	btfss	STATUS, 2		;...continue
	goto continueshutdown		;otherwise, continue the shut down.

	;Now check if the computer is shutdown yet.
	goto checkshutdown	
	
continueshutdown
	;The computer is now off.  Kill power out.
	bcf	GPIO, POWEROUT		;Bit 2 controls the powerout pin.
	
	;Now wait a bit for power to stabilize within the ITPS board before continuing.
	movlw	STABILIZEMINIITXPOWERDELAY
	movwf	DelayTicks
	call	Delay

	;Go to standby.
	goto Standby

;-------------------------------------------------------------------------------

;HardShutdown:
;If the computer gets into an unknown state (such as turned off when it should be
;on), it is necessary to kill power without paying attention to the computer.
;This is the only way to ensure the computer's hardware gets reset.
;HardShutdown is a one way street.  Once it starts, the computer WILL shut down.

HardShutdown

	;The computer should not be on (is not supposed to be anyway).
	;Make a note of it.
	banksel GPIO
	bcf	GPIO, COMPUTERSHOULDBEON	; = 0

	;Kill power.
	bcf	GPIO, POWEROUT

	;Now wait a bit for power to stabilize within the Mini-ITX before continuing.
	movlw	STABILIZEMINIITXPOWERDELAY
	movwf	DelayTicks
	call	Delay

	;The computer is now off.
	;Go to standby.
	;If the car is off, the appropriate budget and snooze algorithms will pick up.
	;If the car is on, it will think the ignition just started.
	goto	Standby


;-------------------------------------------------------------------------------

;Delay:
;Longer delay routine.  Used for delaying a multiple number of ticks (250 ms).
;
;Input: DelayTicks address contains the number of ticks to wait. 
;	Examples:
;	1 tick = 250ms, 
;	4 ticks = 1 second
;	240 ticks = 1 minute.
;	254 ticks is maximum!

Delay
dodelay
	;Delay one tick
	call Delay250ms
	decfsz	DelayTicks, 1
	goto dodelay
	return
	

;-------------------------------------------------------------------------------

;Delay250ms:
;250 ms delay routine

Delay250ms

;	return	;For testing only

	clrwdt		;tickle the watchdog
	movlw	.1	;note count is +1 (decfsz !)
	movwf	waith
	movlw	.243
	movwf	waitm
	clrf	waitl
del1
	nop				;delay loop - 4 us per iteration
	decfsz	waitl,f	
	goto	del1
	decfsz	waitm,f
	  goto	del1
	clrwdt
	decfsz	waith,f
	  goto	del1
	retlw	0
	
	
;-------------------------------------------------------------------------------

;ClearTime
;	Clears out a time structure.
;
;Input: FSR should contain a pointer to the ticks byte of a time structure.

ClearTime
	;Clear the ticks
	clrf	INDF
	;Clear the seconds
	incf	FSR, 1
	clrf	INDF
	;Clear the minutes
	incf	FSR, 1
	clrf	INDF
	;Clear the hours.
	incf	FSR, 1
	clrf	INDF
	
	return	
	
;-------------------------------------------------------------------------------

;IncrementTime
;	Adds one tick to a time structure.

;Input: FSR should contain a pointer to the ticks byte of a time structure.

IncrementTime

	;Before, starting, make sure that the time isn't at the maximum recordable
	;value.  If so, don't increment beyond it.
	movf	INDF, 0			;Ticks = 3?
	xorlw	.3
	btfss	STATUS, 2
	goto	incrementnow1
	incf	FSR, 1			;Seconds = 59?
	movf	INDF, 0
	xorlw	.59
	btfss	STATUS, 2
	goto	incrementnow2
	incf	FSR, 1			;Minutes = 59?
	movf	INDF, 0
	xorlw	0xff
	btfss	STATUS, 2
	goto	incrementnow3
	incf	FSR, 1			;Hours = 0xFF?
	movf	INDF, 0
	xorlw	0xff
	btfss	STATUS, 2
	goto	incrementnow4
	
	;Ticks = 3, Seconds = Minutes = 59, Hours = 0xFF.  So nothing to increment
	return

	;Decrement the FSR again, since it was incremented to check against
	;the maximum counter times.
incrementnow4
	decf	FSR, 1
incrementnow3
	decf	FSR, 1
incrementnow2
	decf	FSR, 1
incrementnow1

	;Ticks
	;Increment the tick byte.
	incf	INDF, 1
	;Check to see if this is the fourth tick.  If not, done.
	btfss	INDF, 2		;Bit 3 = 1 when on the fourth tick.
	return
	;This is the fourth tick.  So zero the ticks...
	clrf	INDF
	;...and increment the seconds.
	incf	FSR, 1
	incf	INDF, 1
	
	;Seconds
	;Now test for the 60th second.
	movfw	INDF
	xorlw	b'00111100'	;60 seconds.
	; If this is the 60th second, increment the minutes.
	btfss	STATUS, 2
	return
	;Zero the seconds...
	clrf	INDF
	;...and increment the minutes.
	incf	FSR, 1
	incf	INDF, 1
	
	;Minutes (Same logic as seconds)
	movfw	INDF
	xorlw	b'00111100'
	btfss	STATUS, 2
	return
	clrf	INDF
	incf	FSR, 1
	incf	INDF, 1
	
	;Hours just increment at this time.
		
	return	

;-------------------------------------------------------------------------------

;DecrementTime
;	Removes one tick from a time structure.
;
;Input: FSR should contain a pointer to the ticks byte of a time structure.

DecrementTime

	;OK, now it is time to decrement the counter.  Because of how the loop is set
	;up below, the counter needs to hold one more than the actual number of loops
	;that it is supposed to do.
	incf	DecrementCounter, 1

	;Before, starting, make sure that the time isn't at the minimum recordable
	;value.  If so, don't decrement below it.
	movf	INDF, 0			;Ticks = 0?
	xorlw	0x00
	btfss	STATUS, 2
	goto	decrementnow1
	incf	FSR, 1			;Seconds = 0?
	movf	INDF, 0
	xorlw	0x00
	btfss	STATUS, 2
	goto	decrementnow2
	incf	FSR, 1			;Minutes = 0?
	movf	INDF, 0
	xorlw	0x00
	btfss	STATUS, 2
	goto	decrementnow3
	incf	FSR, 1			;Hours = 0?
	movf	INDF, 0
	xorlw	0x00
	btfss	STATUS, 2
	goto	decrementnow4
	
	;Ticks = Seconds = Minutes = Hours = 0.  So nothing to decrement
	return

decrementonce

	;Ticks
	;Decrement the tick byte.
	decf	INDF, 1
	;Check to see if this went below the zeroth tick.  If not, done.
	movfw	INDF
	xorlw	b'11111111'	;0xff means underflow (went below zeroth tick)
	btfss	STATUS, 2
	goto	decrementnow1
	;This was the zeroth tick.  So change to tick three...
	movlw	.3
	movwf	INDF
	;...and decrement the seconds.
	incf	FSR, 1
	decf	INDF, 1
	
	;Seconds
	;Now test if this went below the zeroth second.
	movfw	INDF
	xorlw	b'11111111'	;0xFF seconds.
	; If this was the zeroth second, decrement the minutes.
	btfss	STATUS, 2
	goto	decrementnow2
	;This was the zeroth second.  So change to second 59...
	movlw	.59
	movwf	INDF
	;...and decrement the minutes.
	incf	FSR, 1
	decf	INDF, 1
	
	;Minutes (Same logic as seconds)
	movfw	INDF
	xorlw	b'11111111'
	btfss	STATUS, 2
	goto	decrementnow3
	movlw	.59
	movwf	INDF
	incf	FSR, 1
	decf	INDF, 1
	
	;Did this go below the zeroth hour?
	movfw	INDF
	xorlw	b'11111111'
	btfss	STATUS, 2
	goto	decrementnow4
	;Yes, it did go below the zeroth hour.  So just zero out the time.
	movlw	3
	subwf	FSR,1
	call	ClearTime
	return
		
	;Decrement the FSR, since it was incremented in processing the time structure.
decrementnow4
	decf	FSR, 1
decrementnow3
	decf	FSR, 1
decrementnow2
	decf	FSR, 1
decrementnow1

	decfsz	DecrementCounter,1 	;loop again for the next decrement.
	goto decrementonce

	return	

;-------------------------------------------------------------------------------

;CheckAlarm
;	Compares a time structure against an alarm structure to determine if the
;	alarm has triggered.
;
;Input: - FSR should contain a pointer to the ticks byte of a time structure that
;		has an alarm structure attached.
;		- w contains the offset to the alarm.

;The time structure is:
;	Byte 1) Time.Ticks
;	Byte 2) Time.Seconds
;	Byte 3) Time.Minutes
;	Byte 4) Time.Hours
;	Byte 5) Alarm.Seconds
;	Byte 6) Alarm.Minutes
;	Byte 7) Alarm.Hours


CheckAlarm

	;Note the offset of the alarm.
	movwf	AlarmOffset
	movwf	AlarmCounter

	;The first part is to compare the hours.
	;The logic is to subtract: (Alarm.Hours) - (Time.Hours) = ?.
	;If (Alarm.Hours) >= (Time.Hours), carry = 1, and alarm is not triggered.
	;If (Alarm.Hours) < (Time.Hours), carry = 0, and alarm is triggered.
	;Adjust FSR to point to Time.Hours
	movlw	3			;FSR += 3
	addwf	FSR, 1
	;Store Time.Hours in a register for comparison against Alarm.Hours.
	movf	INDF, 0			;w = Time.Hours
	;Adjust FSR to point to Alarm.Hours
hoursalarm1
	incf	FSR, 1			;INDF = Alarm.Hours
	decfsz	AlarmCounter,1
	goto	hoursalarm1
	subwf	INDF, 0			;Alarm.Hours - Time.Hours = w
	btfss	STATUS, 0		;carry = 0 implies Time.Hours > Alarm.Hours
	goto yesalarm			;Time.Hours > Alarm.Hours.  Trigger alarm.
	;Reload AlarmCounter for the next check.
	movf	AlarmOffset, 0
	movwf	AlarmCounter
	;Time.Hours not > Alarm.Hours  Check equal.
	;Adjust FSR to point to Time.Hours again.
	subwf	FSR, 1			;FSR -= AlarmCounter
	movf	INDF,0			;w = Time.Hours
hoursalarm2
	incf	FSR, 1			;INDF = Alarm.Hours
	decfsz	AlarmCounter, 1
	goto	hoursalarm2
	xorwf	INDF, 0			;compare Alarm.Hours and Time.Hours
	btfss	STATUS, 2		;ZeroFlag = 1 implies Alarm.Hours = Time.Hours
	goto	noalarm			;ZeroFlag = 0 implies Time.Hours < Alarm.Hours, 
							;so do not trigger the alarm.
		
	;So far, it is known that Alarm.Hours = Time.Hours
	;Now compare the minutes.
	;Reload AlarmCounter for the next check.
	movf	AlarmOffset, 0
	movwf	AlarmCounter
	;Set the pointers to Time.Minutes.
	addlw	1				;FSR -= (AlarmCounter + 1)
	subwf	FSR, 1
	;Store Time.Minutes in a register for comparison against Alarm.Mins .
	movf	INDF,0			;w = Time.Minutes
	;Adjust FSR to point to Alarm.Minutes
minutesalarm1
	incf	FSR, 1			;INDF = Alarm.Hours
	decfsz	AlarmCounter, 1
	goto	minutesalarm1
	subwf	INDF, 0			;Alarm.Minutes - Time.Minutes = w
	btfss	STATUS, 0		;carry = 0 implies Time.Minutes > Alarm.Minutes
	goto yesalarm			;Time.Minutes > Alarm.Minutes.  Trigger alarm.
	;Reload AlarmCounter for the next check.
	movf	AlarmOffset, 0
	movwf	AlarmCounter
	;The alarm minutes are not greater.  Check equal.
	;Adjust FSR to point to Time.Minutes again.
	subwf	FSR, 1			;FSR -= AlarmCounter (in w)
	movf	INDF,0			;w = Time.Minutes
minutesalarm2
	incf	FSR, 1			;INDF = Alarm.Hours
	decfsz	AlarmCounter, 1
	goto	minutesalarm2
	xorwf	INDF, 0			;compare Alarm.Minutes and Time.Minutes
	btfss	STATUS, 2		;ZeroFlag = 1 implies Alarm.Minutes = Time.Minutes
	goto	noalarm			;ZeroFlag = 0 implies Time.Minutes < Alarm.Minutes, 
							;so do not trigger the alarm.

	;So far, it is known that Alarm.Hours = Time.Hours and Alarm.Minutes = Time.Minutes
	;Now compare the seconds.
	;Reload AlarmCounter for the next check.
	movf	AlarmOffset, 0
	movwf	AlarmCounter
	;Set the pointers to Time.Seconds.
	addlw	1				;FSR -= (AlarmCounter + 1)
	subwf	FSR, 1
	;Store Time.Seconds in a register for comparison against Alarm.Mins .
	movf	INDF,0			;w = Time.Seconds
	;Adjust FSR to point to Alarm.Seconds
secondsalarm1
	incf	FSR, 1			;INDF = Alarm.Hours
	decfsz	AlarmCounter, 1
	goto	secondsalarm1
	subwf	INDF, 0			;Alarm.Seconds - Time.Seconds = w
	btfss	STATUS, 0		;carry = 0 implies Time.Seconds > Alarm.Seconds
	goto yesalarm			;Time.Seconds > Alarm.Seconds.  Trigger alarm.
	;Reload AlarmCounter for the next check.
	movf	AlarmOffset, 0
	movwf	AlarmCounter
	;The alarm seconds are not greater.  Check equal.
	;Adjust FSR to point to Time.Seconds again.
	subwf	FSR, 1			;FSR -= AlarmCounter (in w)
	movf	INDF,0			;w = Time.Seconds
secondsalarm2
	incf	FSR, 1			;INDF = Alarm.Hours
	decfsz	AlarmCounter, 1
	goto	secondsalarm2
	xorwf	INDF, 0			;compare Alarm.Seconds and Time.Seconds
	btfss	STATUS, 2		;ZeroFlag = 1 implies Alarm.Seconds = Time.Seconds
	goto	noalarm			;ZeroFlag = 0 implies Time.Seconds < Alarm.Seconds, 
							;so do not trigger the alarm.
	
	goto	yesalarm		;Hours, minutes, and seconds are equal.  Trigger alarm.
							
noalarm
	retlw	0
	
yesalarm
	retlw	1
	
;-------------------------------------------------------------------------------

; Calibrate the oscillator.  This can be commented out if your programmer
;overwrites the clock.  Otherwise, leave this active.

	org	0x3ff
	retlw	0x94

;Configuration bits for the assmebler:
;	Turns the external reset off.
;	No code protection.
;	Watchdog timer is enabled.
;	Memory protection is off
;	Use Internal RC No Clockout oscillator.
;	Brown-out detect enable.

	__config _MCLRE_OFF & _CP_OFF & _WDT_ON & _PWRTE_ON & _INTRC_OSC_NOCLKOUT
	
	END

